@@ -8,7 +8,7 @@ this ae namespace portion provides useful helper functions for inspection and de
 call stack inspection
 ---------------------
 
-:func:`module_callable` dynamically determines a reference to any callable object (function, class, ...) in a module.
+:func:`module_attr` dynamically determines a reference to an attribute (variable, function, class, ...) in a module.
 
 :func:`module_name`, :func:`stack_frames`, :func:`stack_var` and :func:`stack_vars` are inspecting the call stack frames
 to determine e.g. variable values of the callers of a function/method.
@@ -43,28 +43,28 @@ import sys
 import threading
 import unicodedata
 import weakref
-from _ast import stmt
 
 from importlib.machinery import ModuleSpec
 from inspect import getinnerframes, getouterframes, getsourcefile
 from string import ascii_letters, digits
-from types import ModuleType
-from typing import Any, Callable, Dict, Generator, List, Optional, Tuple, Type
+from typing import Any, Callable, Dict, Generator, List, Optional, Tuple, Type, Union
+from _ast import stmt
 
-from ae.base import DATE_ISO, DATE_TIME_ISO, UNSET                                          # type: ignore
+from ae.base import DATE_ISO, DATE_TIME_ISO, UNSET, UnsetType, norm_name, norm_path, project_main_file  # type: ignore
 
 
-__version__ = '0.2.11'
+__version__ = '0.2.12'
 
 
 # suppress unused import err (needed e.g. for unpickling of dates via try_eval() and for include them into base_globals)
 _d = (DATE_ISO, DATE_TIME_ISO,
-      ascii_letters, digits, datetime, logging, logging_config, threading, unicodedata, weakref)
+      ascii_letters, digits, datetime, logging, logging_config, norm_name, norm_path, threading, unicodedata, weakref)
 
 SKIPPED_MODULES = ('ae.base', 'ae.paths', 'ae.inspector', 'ae.core', 'ae.console', 'ae.gui_app',
                    'ae.gui_help', 'ae.kivy_app', 'ae.enaml_app',    # removed in V 0.1.4: 'ae.lisz_app_data',
                    'ae.beeware_app', 'ae.pyglet_app', 'ae.pygobject_app', 'ae.dabo_app',
-                   'ae.qpython_app', 'ae.appjar_app')
+                   'ae.qpython_app', 'ae.appjar_app',
+                   'importlib._bootstrap', 'importlib._bootstrap_external')
 """ skipped modules used as default by :func:`module_name`, :func:`stack_var` and :func:`stack_vars` """
 
 
@@ -134,36 +134,40 @@ def full_stack_trace(ex: Exception) -> str:
     return ret
 
 
-def module_callable(entry_point: str, module_path: str = "") -> Tuple[Optional[ModuleType], Optional[Callable]]:
-    """ determine dynamically a reference to a module and to a callable declared within the module.
+def module_attr(import_name: str, attr_name: str = "", project_path: Union[str, UnsetType] = "") -> Optional[Any]:
+    """ determine dynamically a reference to a module or to any attribute (variable/func/class) declared in the module.
+
+    :param import_name:         name of the distribution/module/package to load/import. for modules not in the cwd
+                                and not already imported, pass also the :paramref:`~module_attr.project_path` argument.
+    :param attr_name:           name of the attribute declared within the module. if not passed or passed an empty
+                                string then a reference to module instance will be returned.
+    :param project_path:        optional path where the project folder of the searched main module is situated (only
+                                needed if the module is not not in the current working directory and not in `sys.path`/
+                                `sys.modules). to skip the module file search, and directly use sys.modules[import_name]
+                                pass :data:`~ae.base.UNSET`.
+    :return:                    module instance or module attribute value (UNSET if module/attribute doesn't exist).
+
+    -- hint:: a previously not imported module will *not* be added to `sys.modules` by this function.
 
-    :param entry_point:         callable specified in the form <module_name>:<callable_name>. for modules not installed
-                                in `sys.path` pass also the :paramref:`~module_callable.module_path` argument.
-    :param module_path:         optional path where the module is situated (only needed if path is not is `sys.path`).
-    :return:                    tuple of module object and callable object (UNSET if module/callable doesn't exist).
     """
-    module = func = UNSET
-    mod_name, func_name = entry_point.split(':')
-    module_path = os.path.join(module_path, mod_name + '.py')
+    mod_file = "" if project_path is UNSET else project_main_file(import_name, project_path=project_path)
+    mod_ref = UNSET
 
-    if os.path.exists(module_path):
-        spec = importlib.util.spec_from_file_location(mod_name, module_path)
-        # mypy >= V 0.901: had to add import (from importlib.machinery import ModuleSpec) and assert
+    if os.path.isfile(mod_file):
+        spec = importlib.util.spec_from_file_location(import_name, mod_file)
+        # workaround for mypy >= V 0.901: added assert and import (from importlib.machinery import ModuleSpec)
         assert isinstance(spec, ModuleSpec)
-        module = importlib.util.module_from_spec(spec)
+        mod_ref = importlib.util.module_from_spec(spec)
 
         # mypy: had to add import (import importlib.abc) and assert and then also noinspection for PyCharm
         assert isinstance(spec.loader, importlib.abc.Loader)
         # noinspection PyUnresolvedReferences
-        spec.loader.exec_module(module)
+        spec.loader.exec_module(mod_ref)
 
-    elif mod_name in sys.modules:
-        module = sys.modules[mod_name]
+    elif import_name in sys.modules:
+        mod_ref = sys.modules[import_name]
 
-    if module:
-        func = getattr(module, func_name, UNSET)
-
-    return module, func
+    return getattr(mod_ref, attr_name, UNSET) if mod_ref and attr_name else mod_ref
 
 
 def module_file_path(local_object: Optional[Callable] = None) -> str:
@@ -177,7 +181,7 @@ def module_file_path(local_object: Optional[Callable] = None) -> str:
     if local_object:
         file_path = getsourcefile(local_object)
         if file_path:
-            return os.path.abspath(file_path)
+            return norm_path(file_path)
 
     # if getattr(sys, 'frozen', False):
     #    path_without_file = os.getcwd()
@@ -258,7 +262,7 @@ def stack_vars(*skip_modules: str,
     """
     if not skip_modules:
         skip_modules = SKIPPED_MODULES
-    glo = loc = dict()
+    glo = loc = {}
     depth = min_depth + 1   # +1 for stack_frames()
     for frame in stack_frames(depth=depth):
         depth += 1
